#include "painlessMesh.h"

#define   MESH_PREFIX     "prefixMalla"
#define   MESH_PASSWORD   "contrasenyaMalla"
#define   MESH_PORT       7777

#define EXTERNAL_BUTTON_B0 0

Scheduler userScheduler; // to control your personal task
painlessMesh  mesh;

// User stub
void sendMessage() ; // Prototype so PlatformIO doesn't complain

Task taskSendMessage( TASK_SECOND * 1 , TASK_FOREVER, &sendMessage );

void sendMessage() {
  static int n = 0;
  String msg = "";
  // msg += mesh.getNodeId();
  msg += String(n++);
  mesh.sendBroadcast( msg );
  Serial.println( msg );
  taskSendMessage.setInterval( random( TASK_SECOND * 1, TASK_SECOND * 5 ));
}

void sendMessage2(String msg) {
  mesh.sendBroadcast( msg );
}

// Needed for painless library
void receivedCallback( uint32_t from, String &msg ) {
  Serial.printf("startHere: Received from %u msg=%s\n", from, msg.c_str());
  if (msg == "ON" || msg == "on")
    digitalWrite(LED_BUILTIN, HIGH);
  if (msg == "off" || msg == "OFF")
    digitalWrite(LED_BUILTIN, LOW);
}

void newConnectionCallback(uint32_t nodeId) {
  Serial.printf("--> startHere: New Connection, nodeId = %u\n", nodeId);
}

void changedConnectionCallback() {
  Serial.printf("Changed connections\n");
}

void nodeTimeAdjustedCallback(int32_t offset) {
  Serial.printf("Adjusted time %u. Offset = %d\n", mesh.getNodeTime(), offset);
}

boolean bPremut(int nQuinBoto) {
  if (digitalRead(nQuinBoto))
    return false;
  return true;
}

void setup() {
  Serial.begin(115200);
  Serial.println(__FILE__);

  pinMode(LED_BUILTIN, OUTPUT);
  pinMode(EXTERNAL_BUTTON_B0, INPUT_PULLUP);

  //mesh.setDebugMsgTypes( ERROR | MESH_STATUS | CONNECTION | SYNC | COMMUNICATION | GENERAL | MSG_TYPES | REMOTE ); // all types on
  mesh.setDebugMsgTypes( ERROR | STARTUP );  // set before init() so that you can see startup messages

  mesh.init( MESH_PREFIX, MESH_PASSWORD, &userScheduler, MESH_PORT );
  mesh.onReceive(&receivedCallback);
  mesh.onNewConnection(&newConnectionCallback);
  mesh.onChangedConnections(&changedConnectionCallback);
  mesh.onNodeTimeAdjusted(&nodeTimeAdjustedCallback);

  userScheduler.addTask( taskSendMessage );
  taskSendMessage.enable();
}

void loop() {
  static boolean bEstavaPremutBoto00 = false;
  boolean bEstatBOTO_00 = bPremut(EXTERNAL_BUTTON_B0);
  String sMsg;

  if (bEstavaPremutBoto00 != bEstatBOTO_00) {
    delay(10);
    // Hi ha hagut canvi
    if (bEstatBOTO_00) {
      //Serial.println("BOTO_00 premut");
      sMsg = "PRESSED";
    } else
      sMsg = "RELEASED";
    sendMessage2(sMsg);
    Serial.println(sMsg);
    bEstavaPremutBoto00 = bEstatBOTO_00;
  }

  mesh.update();
}
